@@ -17,7 +17,7 @@ database_build_create ()
 	misc_lock_duplicated_executions "${DBDIR}/.lock"
 	if [ -e "${DBDIR}/MYVERSION" ]
 	then
-		if ! grep -m 1 -E "$COMPATIBLE_VERSIONS" "${DBDIR}/MYVERSION" > /dev/null 2>&1
+		if ! grep -q -E "$COMPATIBLE_VERSIONS" "${DBDIR}/MYVERSION" 2> /dev/null
 		then
 			message_echo "ERROR: The current temporary database is incompatible. You must delete it by" >&2
 			message_echo "        ${APPNAME} clean force" >&2
@@ -159,9 +159,16 @@ database_build_setup_make_args ()
 		do
 			eval echo $key=\$$key
 		done
-		echo 'FORCE_PKG_REGISTER=yes'
 		echo 'DISABLE_VULNERABILITIES=yes'
-		[ $opt_apply_default_config = yes ] && echo 'BATCH=yes'
+		if [ $opt_apply_default_config = yes ]
+		then
+			if pkgsys_is_dialog4ports_used
+			then
+				echo 'NO_DIALOG=yes'
+			else
+				echo 'BATCH=yes'
+			fi
+		fi
 		dbdir=${DBDIR}/requires/$origin
 		[ -d "$dbdir" ] || dbdir=${DBDIR}/conf/each_port/$origin
 		cat "$dbdir/MARG.conf" 2> /dev/null || :
@@ -246,7 +253,7 @@ database_build_setup_replace_node ()
 		rm -rf "$dbpath"
 		mkdir -p "$dbpath"
 		origin=$origin_orig
-		if echo "$origin_orig" | grep -m 1 -E -f "${DBDIR}/conf/REPLACE.grep_from_pattern" > /dev/null
+		if echo "$origin_orig" | grep -q -E -f "${DBDIR}/conf/REPLACE.grep_from_pattern"
 		then
 			origin=`echo "$origin_orig" | sed -E -f "${DBDIR}/conf/REPLACE.sed_pattern"`
 			if [ "x$origin_orig" != "x$origin" ]
@@ -318,8 +325,8 @@ database_build_is_port_already_inspected_in_required_level ()
 	[ -n "$origin_actual" ] || origin_actual=$origin
 	origin_esc='^'`str_escape_regexp "$origin_actual"`'$'
 	inspected_level=`database_build_get_inspected_level "$origin_actual" "$origin_dependent"`
-	grep -m 1 -E "^$origin_esc$" "${DBDIR}/ports.inspected.${inspected_level}.list" \
-		"${DBDIR}/obsolete_ports" > /dev/null 2>&1 || return
+	grep -q -E "^$origin_esc$" "${DBDIR}/ports.inspected.${inspected_level}.list" \
+		"${DBDIR}/obsolete_ports" 2> /dev/null || return
 	fileedit_rm_a_line "$origin" "${DBDIR}/stage.loop_list/ports_to_inspect.remain"
 	[ "x$origin" = "$origin_actual" ] || \
 		fileedit_rm_a_line "$origin_actual" "${DBDIR}/stage.loop_list/ports_to_inspect.remain"
@@ -404,9 +411,9 @@ database_build_update_pkgtag ()
 {
 	local origin pkg_init pkg_bak pkg_cur detail pkgtag
 	origin=$1
-	pkg_init=`cat "${DBDIR}/requires/$origin/installed_version" 2> /dev/null || :`
-	pkg_bak=`cat "${DBDIR}/requires/$origin/backedup_version" 2> /dev/null || :`
-	pkg_cur=`database_build_get_pkgname "$origin"`
+	pkg_init=`cat "${DBDIR}/requires/$origin/installed_version" 2> /dev/null | tr '\n' ' ' | sed 's/ *$//'`
+	pkg_bak=`cat "${DBDIR}/requires/$origin/backedup_version" 2> /dev/null | tr '\n' ' ' | sed 's/ *$//'`
+	pkg_cur=`database_build_get_pkgname "$origin" | tr '\n' ' ' | sed 's/ *$//'`
 	detail=
 	if [ "x$pkg_init" != "x$pkg_cur" ]
 	then
@@ -465,7 +472,7 @@ database_build_inspect_dependencies ()
 		if [ ! -e "$dbpath/complete_as_node" ]
 		then
 			conf_updated=
-			if grep -m 1 -E -e "^$origin_orig_regexp$" -e "^$origin_regexp$" "${DBDIR}/to_be_reconf" > /dev/null 2>&1
+			if grep -q -E -e "^$origin_orig_regexp$" -e "^$origin_regexp$" "${DBDIR}/to_be_reconf" 2> /dev/null
 			then
 				message_echo "${DEPTH_INDEX}  ===> Reconfigured"
 				conf_updated=y
@@ -490,17 +497,8 @@ database_build_inspect_dependencies ()
 			then
 				if [ $opt_apply_default_config = yes ]
 				then
-					if pkgsys_is_dialog4ports_used
+					if ! pkgsys_is_dialog4ports_used
 					then
-						if [ -n "$SCRIPT" ]
-						then
-							message_echo "ERROR: script(1) prevents automatic configurations of port options via dialog4ports(1) (at least found for dialog4ports-0.1.3)." >&2
-							message_echo "If you want to enable the automatic configurations, rerun outside of script(1) environment at least until the end of section [Inspecting dependencies of the all installed packages]." >&2
-							message_echo "If you can accept interactive configurations, disable -C option with the aid of -M option." >&2
-							exit 1
-						fi
-						printf '\n' | database_build_make "$origin" config-conditional
-					else
 						printf '\t\n' | database_build_make "$origin" config-conditional > /dev/null
 					fi
 				else
@@ -572,8 +570,8 @@ database_build_inspect_dependencies ()
 		do
 			origin_dependency=`sed -n ${iline}p "${TMPDIR}/missing.$origin_id"`
 			iline=$(($iline+1))
-			grep -m 1 -E '^'`str_escape_regexp "$origin_dependency"`'$' \
-				"${DBDIR}/ports.inspected.list" > /dev/null 2>&1 && \
+			grep -q -E '^'`str_escape_regexp "$origin_dependency"`'$' \
+				"${DBDIR}/ports.inspected.list" 2> /dev/null && \
 					continue
 			database_build_inspect_dependencies "$origin_dependency" "$origin"
 		done
@@ -618,7 +616,7 @@ database_build_get_complete_recursive_dependency ()
 	dbpath=${DBDIR}/requires/$origin
 	[ -e "$dbpath/$table.direct" ] || return 0
 	dstfile=$dbpath/$table.full
-	if [ ! -e "$dbpath/$table" -a ! -e "$dstfile" ]
+	if [ ! -e "$dstfile" ]
 	then
 		while read origin_requirement
 		do
@@ -899,6 +897,7 @@ database_build_patch_reconf ()
 # ============= Post`processes after finishing to inspect dependencies =============
 database_build_post_inspect_dependencies ()
 {
+	local table
 	touch "${DBDIR}/obsolete_ports" "${DBDIR}/inspected_ports.update"
 	find "${DBDIR}/requires" -depth 2 -type d \
 		| sed -E 's|.*/([^/]+/[^/]+)$|\1|' > ${DBDIR}/inspected_ports
@@ -955,5 +954,16 @@ database_build_post_inspect_dependencies ()
 		program_chk_stage_loop_complete CONVERT_REQUIREMENTS_LIST \
 			&& rm -f "${DBDIR}/stage.loop_list/convert_dependency_lists"
 	fi
+	{
+		sed -E -f "${DBDIR}/REPLACE.complete_sed_pattern" "${DBDIR}/need.list" || :
+		cat "${DBDIR}/need.list" || :
+	} 2> /dev/null | sort -u > ${DBDIR}/need.with_replaced.list
+	for table in requirements dependents itself
+	do
+		[ -e "${DBDIR}/stage.loop_list/target_$table.specified" ] || continue
+		sed -E -f "${DBDIR}/REPLACE.complete_sed_pattern" \
+			"${DBDIR}/stage.loop_list/target_$table.specified" \
+			> ${DBDIR}/stage.loop_list/target_$table.replaced.specified
+	done	
 	cp /dev/null "${DBDIR}/update_dependencies"
 }
