@@ -27,8 +27,8 @@ database_record_success ()
 				sed "s/^/$dbdir_esc/;s|$|/failed_requirements.$dbsuffix|" "$nodedir/dependents.$dbsuffix" | \
 					fileedit_rm_a_line_from_files "$origin"
 			fi
-			if [ ! -e "$nodedir/installed_by_pkg" ] && grep -m 1 -E -f "$tmp_grep_failed" \
-				"$nodedir/requirements.$dbsuffix" > /dev/null 2>&1
+			if [ ! -e "$nodedir/installed_by_pkg" ] && grep -q -E -f "$tmp_grep_failed" \
+				"$nodedir/requirements.$dbsuffix" 2> /dev/null
 			then
 				fileedit_rm_a_line "$origin" "${DBDIR}/success.$dbsuffix.list"
 				fileedit_add_a_line_if_new "$origin" "${DBDIR}/todo_after_requirements_succeed.$dbsuffix.list"
@@ -41,6 +41,7 @@ database_record_success ()
 		done
 	done
 	fileedit_rm_a_line "$origin" "${DBDIR}/failed.list"
+	fileedit_rm_a_line "$origin" "${DBDIR}/damaged_package"
 	touch "$nodedir/succeeded_once"
 	rm -f "$nodedir/note_failtre" "$nodedir/conf_updated"
 }
@@ -77,3 +78,21 @@ database_record_failure ()
 	message_echo "*** Skipping this port and proceeding to next forcibly..."
 	message_echo
 }
+
+# ============= Record a port that it was damaged by losing or destroying installed files =============
+database_record_reconf_recover_sanity ()
+{
+	local origin tag level dbsuffix
+	origin=$1
+	fileedit_add_a_line_if_new "$origin" "${DBDIR}/damaged_package"
+	for tag in all run build none
+	do
+		for level in full direct
+		do
+			dbsuffix=$tag.$level
+			fileedit_rm_a_line "$origin" "${DBDIR}/success.$dbsuffix.list"
+			fileedit_rm_a_line "$origin" "${DBDIR}/todo_after_requirements_succeed.$dbsuffix.list"
+		done
+	done
+	rm -f "${DBDIR}/requires/$origin/succeeded_once"
+}
