@@ -21,7 +21,7 @@ deinstall_select_leaf_ports_to_delete_dialog ()
 	do
 		pkgtag=`cat "${DBDIR}/requires/$origin/pkgtag"`
 		val=on
-		grep -m 1 '^'`str_escape_regexp "$origin"`'$' "$dst_unselected" > /dev/null && val=off
+		grep -q '^'`str_escape_regexp "$origin"`'$' "$dst_unselected" 2> /dev/null && val=off
 		printf '%s\t"(%s)"\t%s\n' "$origin" "$pkgtag" "$val"
 	done < $src > $itemlist
 	misc_dialog_checklist "$title" "$desc" "$dst_selected" "$itemlist"
@@ -47,8 +47,8 @@ deinstall_select_auto ()
 	fileedit_manipulate_old_new_lines "$tmp_srcprev" "$tmp_srcnew" "$diff_removed" "$diff_added" || return 0
 	str_escape_regexp_filter < $diff_removed | sed 's/^/^/;s/$/$/' > $diff_removed_ptn
 	str_escape_regexp_filter < $src | sed 's/^/^/;s/$/$/' > $diff_available_ptn
-	grep -v -E -f "$diff_removed_ptn" "$dst_rem" | \
-		grep -E -f "$diff_available_ptn" > $dst_rem.tmp 2> /dev/null || :
+	grep -v -E -f "$diff_removed_ptn" "$dst_rem" 2> /dev/null | \
+		grep -E -f "$diff_available_ptn" > $dst_rem.tmp || :
 	mv "$dst_rem.tmp" "$dst_rem"
 	cat "$dst_add" "$diff_added" 2> /dev/null | sort -u | \
 		grep -E -f "$diff_available_ptn" > $dst_add.tmp
@@ -111,7 +111,7 @@ deinstall_select_obsolete_ports_to_delete_dialog ()
 	do
 		pkgtag=`cat "${DBDIR}/initial/$origin/installed_version"`
 		val=on
-		grep -m 1 '^'`str_escape_regexp "$origin"`'$' "$dst_unselected" > /dev/null && val=off
+		grep -q '^'`str_escape_regexp "$origin"`'$' "$dst_unselected" 2> /dev/null && val=off
 		printf '%s\t"(%s)"\t%s\n' "$origin" "$pkgtag" "$val"
 	done < $src > $itemlist
 	misc_dialog_checklist "$title" "$desc" "$dst_selected" "$itemlist"
@@ -159,11 +159,14 @@ deinstall_select_obsolete_ports_to_delete ()
 # ============= Restoration of a package for an origin =============
 deinstall_restore ()
 {
-	local origin currentpkg backup_pkg
+	local origin tmp_forbidden currentpkg backup_pkg pkg
 	origin=$1
+	tmp_forbidden=${TMPDIR}/deinstall_restore:forbidden
 	pkg_info_eO "$origin" && return
 	backup_pkg=`pkgsys_get_backup_pkg "$origin"` || return 0
 	currentpkg=`pkgsys_pkgarc_to_pkgname "$backup_pkg"`
+	message_stage_title "$PROGRAM_STEP_COUNTER $origin ($currentpkg)"
+	message_target_relations "$origin"
 	temp_set_msg_current_stage "a ${_MSG_CURRENT_STAGE_general} process for $origin ($currentpkg) $PROGRAM_STEP_COUNTER"
 	database_query_get_target_attributes currentorigin "$origin"
 	if [ -z "${currentorigin_is_relevant}" ]
@@ -180,8 +183,12 @@ deinstall_restore ()
 		temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
 		return
 	fi
-	message_stage_title "$PROGRAM_STEP_COUNTER $origin ($currentpkg)"
-	message_target_relations "$origin"
+	pkg=`pkgsys_pkgarc_to_pkgname "$backup_pkg"`
+	if reinstall_chk_forbidden_conflicts "$pkg"
+	then
+		message_echo "-- (Skipped because conflicting with installed packages)"
+		return
+	fi
 	if [ $opt_dry_run = yes ]
 	then
 		message_dry_run
@@ -199,12 +206,12 @@ deinstall_restore ()
 # ============= Deinstallation of a package for an origin =============
 deinstall_exec ()
 {
-	local origin origin_regexp currentpkg backup_pkgdir backup_pkg
+	local origin origin_regexp currentpkg backup_pkgdir backup_pkg pkg
 	origin=$1
 	origin_regexp=`str_escape_regexp "$origin"`
 	grep -v -E "^${origin_regexp}[[:space:]]" "${DBDIR}/deleted_conflicts" > ${DBDIR}/deleted_conflicts.tmp || :
 	mv "${DBDIR}/deleted_conflicts.tmp" "${DBDIR}/deleted_conflicts"
-	currentpkg=`pkg_info_qO "$origin" 2> /dev/null`
+	currentpkg=`pkg_info_qO "$origin" 2> /dev/null | tr '\n' ' ' | sed 's/ *$//'`
 	[ -n "$currentpkg" ] || return 0
 	temp_set_msg_current_stage "a ${_MSG_CURRENT_STAGE_general} process for $origin ($currentpkg) $PROGRAM_STEP_COUNTER"
 	database_query_get_target_attributes currentorigin "$origin"
@@ -241,7 +248,8 @@ deinstall_exec ()
 			backup_pkgdir=${PKGREPOSITORY}
 		fi
 		[ -d "$backup_pkgdir" ] || mkdir -p "$backup_pkgdir"
-		backup_pkg=`pkgsys_create_backup_pkg "$currentpkg" "$backup_pkgdir"` || :
+		pkg=`echo "$currentpkg" | tr ' ' '\n' | grep -v '^$' | tail -n 1`
+		backup_pkg=`pkgsys_create_backup_pkg "$pkg" "$backup_pkgdir"` || :
 		if [ -n "$backup_pkg" ]
 		then
 			message_echo "INFO: The backup is saved at $backup_pkg."
@@ -254,13 +262,13 @@ deinstall_exec ()
 		fi
 	fi
 	message_echo "-- (Deleting package for $origin as $currentpkg)"
-	if ! pkg_delete_f "$currentpkg"
-	then
-		message_echo "*** Continuating forcibly by hoping success..."
-		message_echo
-		temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
-		return
-	fi
+	echo "$currentpkg" | tr ' ' '\n' | grep -v '^$' | while read pkg
+	do
+		if ! pkg_delete_f "$currentpkg"
+		then
+			message_echo "*** Continuating forcibly by hoping success..."
+		fi
+	done
 	temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
 	message_echo
 }
