@@ -33,7 +33,7 @@ fileedit_add_a_line_if_new ()
 	if [ -e "$dstpath" ]
 	then
 		cp "$dstpath" "$tmpfile"
-		grep -m 1 -E "^$pattern$" "$dstpath" > /dev/null  2>&1 \
+		grep -q -E "^$pattern$" "$dstpath" \
 			|| echo "$item" >> $tmpfile
 	else
 		dstdir=`dirname "$dstpath"`
@@ -79,7 +79,7 @@ fileedit_add_lines_if_new ()
 	then
 		while read origin
 		do
-			grep -m 1 -E "^`str_escape_regexp \"$origin\"`$" "$dstpath" > /dev/null 2>&1 \
+			grep -q -E "^`str_escape_regexp \"$origin\"`$" "$dstpath" \
 				|| echo "$origin"
 		done > $tmpclip
 		if [ "@$advance" = @advance ]
@@ -154,5 +154,5 @@ fileedit_exists_old_lines ()
 	[ -e "$oldsrc" ] || oldsrc=/dev/null
 	[ -e "$newsrc" ] || newsrc=/dev/null
 	diff "$oldsrc" "$newsrc" > $tmpdiff && return 1
-	grep -m 1 '^<' "$tmpdiff" > /dev/null
+	grep -q '^<' "$tmpdiff"
 }
