@@ -222,7 +222,7 @@ pkgsys_def_pkgtools ()
 		PKGSYS_USE_PKGNG=`cat "${DBDIR}/WITH_PKGNG"`
 	else
 		PKGSYS_USE_PKGNG=`pkgsys_sysvar WITH_PKGNG | tr '[:upper:]' '[:lower:]'`
-		if [ -d "${DBDIR}" ]
+		if [ -d "${DBDIR}" ] && misc_is_superuser_privilege
 		then
 			echo "$PKGSYS_USE_PKGNG" > ${DBDIR}/WITH_PKGNG.tmp
 			mv "${DBDIR}/WITH_PKGNG.tmp" "${DBDIR}/WITH_PKGNG"
@@ -237,7 +237,11 @@ pkgsys_def_pkgtools ()
 		PKGSYS_CMD_PKG_ADD='pkg add'
 		pkg_is_tool_available ()
 		{
-			which -a pkg | grep -v '^/usr/sbin/pkg$' | grep -m 1 '/sbin/pkg$' > /dev/null 2>&1
+			which -a pkg | grep -v '^/usr/sbin/pkg$' | grep -q '/sbin/pkg$'
+		}
+		pkg_info_Ea ()
+		{
+			pkg info -qa 2> /dev/null
 		}
 		pkg_info_qoa ()
 		{
@@ -283,6 +287,22 @@ pkgsys_def_pkgtools ()
 		{
 			pkg info -Ex "$@" 2> /dev/null
 		}
+		pkg_info_qL ()
+		{
+			pkg info -ql "$@" 2> /dev/null
+		}
+		pkg_check_sanity ()
+		{
+			local pkg
+			pkg=$1
+			pkg check -s "$pkg" 2> /dev/null
+		}
+		pkg_which ()
+		{
+			local filepath
+			filepath=$1
+			pkg which -q "$filepath" 2> /dev/null
+		}
 		pkg_info_gen_pkg_origin_table ()
 		{
 			pkg query -g '%n-%v\t%o' \* 2> /dev/null > ${DBDIR}/installed_ports:pkg_vs_origin.tbl
@@ -337,13 +357,15 @@ pkgsys_def_pkgtools ()
 		{
 			pkg_add_f "$@"
 		}
-		pkg_inst_remote ()
+		pkg_inst_remote_fetch ()
 		{
-			local pkg pkgarc
+			local pkg mode pkgarc
 			pkg=$1
+			mode=$2
 			tmp_extract=${TMPDIR}/pkgng:pkg_inst_remote:extract
 			pkg fetch -yU "$pkg" || return
 			pkgarc=`pkgsys_pkgname_to_pkgarc "${PKGNG_PKG_CACHEDIR}/All" "$pkg"` || return
+			[ "x$mode" = xnodepschk ] && return
 			rm -rf "$tmp_extract"
 			mkdir "$tmp_extract"
 			tar xf "$pkgarc" -C "$tmp_extract" +MANIFEST
@@ -357,24 +379,39 @@ pkgsys_def_pkgtools ()
 					return 1
 				fi
 			done
+			:
+		}
+		pkg_inst_remote ()
+		{
+			local pkg mode pkgarc
+			pkg=$1
+			mode=$2
+			pkg_inst_remote_fetch "$pkg" "$mode" || return
+			pkgarc=`pkgsys_pkgname_to_pkgarc "${PKGNG_PKG_CACHEDIR}/All" "$pkg"` || return
 			env ASSUME_ALWAYS_YES=YES pkg add "$pkgarc"
 		}
-		pkg_inst_remote_wild ()
+		pkg_inst_remote_wild_fetch ()
 		{
 			local pkg mode pkgarc
 			pkg=$1
 			mode=$2
 			if pkg_is_tool_available
 			then
-				pkg_inst_remote "$pkg" && return
+				pkg_inst_remote "$pkg" "$mode" && return
 			fi
 			message_echo "INFO: Trying to use a legacy package and convert it to pkgng."
 			pkgsys_fetch_legacy_remote "$pkg" || return
 			pkgarc=`pkgsys_fetch_legacy_remote__fetched_pkg`
-			if [ "x$mode" != xnodepschk ]
-			then
-				pkg_is_dependency_of_a_legacypkg_latest "$pkgarc" || return
-			fi
+			[ "x$mode" = xnodepschk ] && return
+			pkg_is_dependency_of_a_legacypkg_latest "$pkgarc"
+		}
+		pkg_inst_remote_wild ()
+		{
+			local pkg mode pkgarc
+			pkg=$1
+			mode=$2
+			pkg_inst_remote_wild_fetch "$pkg" "$mode" || return
+			pkgarc=`pkgsys_fetch_legacy_remote__fetched_pkg`
 			pkg_add -ifF "$pkgarc" || return
 			message_echo "INFO: Trying to convert the installed legacy package to pkgng."
 			pkg2ng || :
@@ -441,6 +478,10 @@ pkgsys_def_pkgtools ()
 		{
 			:
 		}
+		pkg_info_Ea ()
+		{
+			pkg_info -Ea 2> /dev/null
+		}
 		pkg_info_qoa ()
 		{
 			pkg_info -qoa 2> /dev/null
@@ -485,6 +526,22 @@ pkgsys_def_pkgtools ()
 		{
 			pkg_info -Ex "$@" 2> /dev/null
 		}
+		pkg_info_qL ()
+		{
+			pkg_info -qL "$@" 2> /dev/null
+		}
+		pkg_check_sanity ()
+		{
+			local pkg
+			pkg=$1
+			pkg_info -qg "$pkg" 2> /dev/null
+		}
+		pkg_which ()
+		{
+			local filepath
+			filepath=$1
+			pkg_info -qW "$filepath" 2> /dev/null
+		}
 		pkg_info_gen_pkg_origin_table ()
 		{
 			pkg_info -aE 2> /dev/null | while read pkg
@@ -509,18 +566,32 @@ pkgsys_def_pkgtools ()
 		{
 			pkg_add -ifF "$@"
 		}
-		pkg_inst_remote ()
+		pkg_inst_remote_fetch ()
 		{
-			local pkg pkgarc
+			local pkg mode pkgarc
 			pkg=$1
+			mode=$2
 			pkgsys_fetch_legacy_remote "$pkg" || return
 			pkgarc=`pkgsys_fetch_legacy_remote__fetched_pkg`
-			pkg_is_dependency_of_a_legacypkg_latest "$pkgarc" || return
+			[ "x$mode" = xnodepschk ] && return
+			pkg_is_dependency_of_a_legacypkg_latest "$pkgarc"
+		}
+		pkg_inst_remote ()
+		{
+			local pkg mode pkgarc
+			pkg=$1
+			mode=$2
+			pkg_inst_remote_fetch "$pkg" "$mode" || return
+			pkgarc=`pkgsys_fetch_legacy_remote__fetched_pkg`
 			pkg_add "$pkgarc" || return
 		}
+		pkg_inst_remote_wild_fetch ()
+		{
+			pkg_inst_remote_fetch "$1" "$2"
+		}
 		pkg_inst_remote_wild ()
 		{
-			pkg_inst_remote "$1"
+			pkg_inst_remote "$1" "$2"
 		}
 		pkg_loadconf () { :; }
 		pkg_rescue_tools () { :; }
@@ -534,7 +605,7 @@ pkgsys_pkg_info_eO ()
 	origin=$1
 	origin_regexp=`str_escape_regexp "$origin"`
 	cut -f 2 "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
-		| grep -m 1 -E "^$origin_regexp$" > /dev/null && return
+		| grep -q -E "^$origin_regexp$" && return
 	pkg_info_eO "$origin"
 }
 
@@ -652,7 +723,7 @@ pkgsys_eval_ports_glob ()
 			{
 				pkg_info_qoa
 				cut -f 2 "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null
-			} | grep -E "$glob" 2>&1 || :
+			} | grep -E "$glob_regexp" 2>&1 || :
 		else
 			if expr "$glob" : '^[a-z][a-zA-Z0-9+-]*[a-zA-Z0-9+]$' > /dev/null 2>&1 && \
 				[ `expr "$glob" : '.*-[0-9]' 2>&1` -eq 0 ]
@@ -774,7 +845,7 @@ pkgsys_pkgname_to_pkgarc ()
 	if [ "x$PKGSYS_USE_PKGNG" = xyes ]
 	then
 		pkgname_ptn=`str_escape_regexp "$pkgname"`
-		pkgnode=`ls "$pkgdir" 2> /dev/null | grep -m 1 -E "^${pkgname_ptn}\.(txz|tbz|tgz|tar)$"` :
+		pkgnode=`ls "$pkgdir" 2> /dev/null | grep -m 1 -E "^${pkgname_ptn}\.(txz|tbz|tgz|tar)$"` || :
 	elif [ -e "$pkgdir/$pkgname.tbz" ]
 	then
 		pkgnode=$pkgname.tbz
@@ -837,10 +908,10 @@ pkgsys_eval_installed_pkgs_globs ()
 	done < $tmp_evaluated
 }
 
-# ============= Get conflicting packages of a port =============
-pkgsys_get_conflicting_pkgs ()
+# ============= Get glob patterns of conflicting packages of a port =============
+pkgsys_get_conflicting_pkgs_patterns ()
 {
-	local origin mode conflicts
+	local mode origin conflicts
 	mode=$1
 	origin=$2
 	conflicts=`database_query_get_makevar_val "$origin" CONFLICTS`
@@ -852,7 +923,87 @@ pkgsys_get_conflicting_pkgs ()
 		conflicts=$conflicts' '`database_query_get_makevar_val "$origin" CONFLICTS_INSTALL`
 		;;
 	esac
-	pkg_info_Eg $conflicts > ${TMPDIR}/pkgsys_get_conflicting_pkgs::conflicts || :
-	cat "${TMPDIR}/pkgsys_get_conflicting_pkgs::conflicts"
-	[ `wc -l < ${TMPDIR}/pkgsys_get_conflicting_pkgs::conflicts` -gt 0 ]
+	echo "$conflicts" | tr ' ' '\n' | grep -v '^$' || :
 }	
+
+# ============= Get conflicting installed packages of a port =============
+pkgsys_get_conflicting_installed_pkgs ()
+{
+	local mode origin tmp_conflicts
+	mode=$1
+	origin=$2
+	tmp_conflicts=${TMPDIR}/pkgsys_get_conflicting_installed_pkgs::conflicts
+	pkg_info_Eg `pkgsys_get_conflicting_pkgs_patterns "$mode" "$origin"` > $tmp_conflicts || :
+	cat "$tmp_conflicts"
+	[ `wc -l < $tmp_conflicts` -gt 0 ]
+}	
+
+# ============= Check whether a package conflicts with a port =============
+pkgsys_chk_conflict_by_a_pkg ()
+{
+	local mode origin pkg tmp_conflicts_ptn
+	mode=$1
+	origin=$2
+	pkg=$3
+	tmp_conflicts_ptn=${TMPDIR}/pkgsys_chk_conflict_by_a_pkg::conflicts_ptn
+	pkgsys_get_conflicting_pkgs_patterns "$mode" "$origin" \
+		| str_convert_glob_to_regexp_pattern > $tmp_conflicts_ptn
+	echo "$pkg" | grep -q -E -f "$tmp_conflicts_ptn"
+}	
+
+# ============= Check whether installed files are lost or broken for a package =============
+pkgsys_sanitychk_pkgcontents ()
+{
+	local pkg var_is_reinstall_encouraged _is_reinstall_encouraged tmp_sanity nlines iline src filename icol filename_esc pkg_owner origin
+	pkg=$1
+	var_is_reinstall_encouraged=$2
+	tmp_sanity=${TMPDIR}/pkgsys_sanitychk_pkgcontents:sanity
+	pkg_check_sanity "$pkg" > $tmp_sanity || :
+	eval "$var_is_reinstall_encouraged=no"
+	[ `wc -l < $tmp_sanity` -eq 0 ] && return
+	nlines=`wc -l < $tmp_sanity`
+	iline=1
+	_is_reinstall_encouraged=no
+	while [ $iline -le $nlines ]
+	do
+		src=`sed -n ${iline}p "$tmp_sanity"`
+		iline=$(($iline+1))
+		filename=`echo "$src" | cut -d ' ' -f 1`
+		icol=2
+		until [ -e "$filename" -o "$filename" = "$src" ]
+		do
+			filename="$filename "`echo "$src" | cut -d ' ' -f $icol`
+		done
+		[ -e "$filename" ] || continue
+		if expr "$filename" : '.*/include/.*' > /dev/null
+		then
+			_is_reinstall_encouraged=yes
+			break
+		fi
+		filename_esc=`str_escape_regexp "$filename"`
+		if file "$filename" | sed -E "s/^$filename_esc:[[:space:]]//" | grep -q '^ELF '
+		then
+			_is_reinstall_encouraged=yes
+			break
+		fi
+		pkg_owner=`pkg_which "$filename"`
+		if [ "$pkg" != "$pkg_owner" ]
+		then
+			_is_reinstall_encouraged=yes
+			break
+		fi
+	done
+	eval "$var_is_reinstall_encouraged=\$_is_reinstall_encouraged"
+	origin=`pkg_info_qo "$pkg"`
+	if [ $opt_batch_mode = no ]
+	then
+		message_echo "[$pkg ($origin)]"
+		sed 's/^/  /' "$tmp_sanity"
+		message_echo
+	else
+		pkg_replace=`str_escape_replaceval "$pkg"`
+		origin_replace=`str_escape_replaceval "$origin"`
+		sed "s/^/$pkg_replace\\\\$origin_replace\\\\$_is_reinstall_encouraged\\\\/" "$tmp_sanity" | tr '\\' '\t'
+	fi
+	return 1
+}
