@@ -165,7 +165,7 @@ program_exec_restartable_loop_operation ()
 		iline=$(($iline+1))
 		PROGRAM_STEP_PROGRESS=$(($PROGRAM_STEP_PROGRESS+1))
 		PROGRAM_STEP_COUNTER="[$PROGRAM_STEP_PROGRESS/$PROGRAM_NUM_STEPS $(($PROGRAM_STEP_PROGRESS*100/$PROGRAM_NUM_STEPS))%]"
-		grep -m 1 -E "^`str_escape_regexp \"$item\"`$" "$looplist.remain" > /dev/null || continue
+		grep -q -E "^`str_escape_regexp \"$item\"`$" "$looplist.remain" 2> /dev/null || continue
 		_program_exec_restartable_loop_operation__routine "$item"
 		fileedit_rm_a_line "$item" "$looplist.remain"
 	done
