@@ -12,6 +12,7 @@ REINSTALL_PKGTAG=	# Tag indicating version changes by reinstallation of the curr
 REINSTALL_ORIGPKGTAG=	# Tag indicating the port origin and version changes by reinstallation of the current port
 REINSTALL_CURRENTPKG=	# Currently installed package name of the current port
 REINSTALL_ORIGIN=	# Port origin of the current port
+REINSTALL_ORIGIN_REGEXP=	# Port origin of the current port where control characters of extended regular expression are escaped
 REINSTALL_DBNODE_DIR=	# Path of the "requires" section of the temporary database for the current port
 REINSTALL_IS_CURRENTPKG_LATEST=	# Currently installed package name of the current port
 REINSTALL_DBSUFFIX=	# Database suffix for the evaluation method of dependencies
@@ -27,14 +28,31 @@ reinstall_skip ()
 	fileedit_rm_a_line "$REINSTALL_ORIGIN" "${DBDIR}/stage.loop_list/reinst_todo.remain"
 }
 
+# ============= Check whether a package is forbidden due to conflicts =============
+reinstall_chk_forbidden_conflicts ()
+{
+	local pkg tmp_forbidden backup_pkg pkg pkg_regexp_esc
+	pkg=$1
+	tmp_forbidden=${TMPDIR}/reinstall_chk_forbidden_conflicts:forbidden
+	pkg_regexp_esc=`str_escape_regexp "$pkg"`
+	grep -E "^$pkg_regexp_esc:" "${DBDIR}/forbidden_conflicts" > $tmp_forbidden 2> /dev/null || return
+	pkg_info_e `cut -d : -f 2 "$tmp_forbidden"`
+}
+
 # ============= Restore a package if it is temporarily deinstalled =============
 reinstall_restore_if_temporarily_deinstalled ()
 {
-	local backup_pkg
+	local backup_pkg pkg
 	[ -n "$REINSTALL_CURRENTPKG" ] && return
 	backup_pkg=`cat "$REINSTALL_DBNODE_DIR/backedup_pkgfile" 2> /dev/null` || return 0
 	[ -z "$backup_pkg" -o ! -e "$backup_pkg" ] && return
-	message_echo "INFO: Temporarily deinstalled, so restoring from the backup."
+	pkg=`pkgsys_pkgarc_to_pkgname "$backup_pkg"`
+	if reinstall_chk_forbidden_conflicts "$pkg"
+	then
+		message_echo "INFO: This port is temporarily deinstalled but avoided because it conflicts with installed packages."
+		return
+	fi
+	message_echo "INFO: This port is temporarily deinstalled, so restoring from the backup."
 	pkg_add_fF "$backup_pkg" && return
 	message_echo "WARNING: Failed to restore the backup, but continuing anyway." >&2
 }
@@ -47,7 +65,7 @@ reinstall_skip_if_in_a_list ()
 	list=$2
 	mode=$3
 	origin_regexp_esc=`str_escape_regexp "$REINSTALL_ORIGIN"`
-	grep -m 1 -E "^$origin_regexp_esc$" "${DBDIR}/$list" > /dev/null 2>&1 || return
+	grep -q -E "^$origin_regexp_esc$" "${DBDIR}/$list" 2> /dev/null || return
 	[ "x$mode" = xrestore ] && reinstall_restore_if_temporarily_deinstalled
 	reinstall_skip "$message" || :
 }
@@ -62,7 +80,6 @@ reinstall_setup_make_args ()
 		do
 			eval echo $key=\$$key
 		done
-		echo 'FORCE_PKG_REGISTER=yes'
 		[ $opt_avoid_vulner = yes ] || echo 'DISABLE_VULNERABILITIES=yes'
 		case $mode in
 		anymode);;
@@ -71,6 +88,10 @@ reinstall_setup_make_args ()
 			[ $opt_interactive_ports_only = yes ] && echo 'INTERACTIVE=yes'
 			;;
 		esac
+		if [ $opt_apply_default_config = yes ]
+		then
+			pkgsys_is_dialog4ports_used && echo 'NO_DIALOG=yes'
+		fi
 		cat "${DBDIR}/requires/$REINSTALL_ORIGIN/MARG.conf" 2> /dev/null || :
 	} | tr '\n' ' '
 }
@@ -229,12 +250,37 @@ reinstall_restore_conflicts ()
 			pkg_info_e "$pkg" || pkg_info_eO "$origin" || \
 			{ [ -n "$origin_orig" ] && pkg_info_eO "$origin_orig"; } || \
 			{ [ -n "$origin_replace" ] && pkg_info_eO "$origin_replace"; }
+		then
+			is_to_dereg_from_list=yes
+		else
+			is_to_dereg_from_list=no
+		fi
+		if pkgsys_chk_conflict_by_a_pkg install "$REINSTALL_ORIGIN" "$pkg"
+		then
+			fileedit_add_a_line_if_new "$pkg:$REINSTALL_NEWPKGNAME" "${DBDIR}/forbidden_conflicts"
+			is_skipped=yes
+		elif reinstall_chk_forbidden_conflicts "$pkg"
+		then
+			message_echo "INFO: Restoration of a conflict, $origin ($pkg), is avoided because it conflicts with installed packages."
+			is_skipped=yes
+		else
+			is_skipped=no
+		fi
+		if [ $is_to_dereg_from_list = yes ]
 		then
 			grep -v -E "[[:space:]]$pkg_regexp_esc$" "${DBDIR}/deleted_conflicts" \
 				> ${DBDIR}/deleted_conflicts.tmp || :
 			mv "${DBDIR}/deleted_conflicts.tmp" "${DBDIR}/deleted_conflicts"
 			continue
 		fi
+		[ $is_skipped = yes ] && continue
+		origin_regexp_esc=`str_escape_regexp "$origin"`
+		origin_replace_regexp_esc=`str_escape_regexp "$origin_replace"`
+		if grep -q -E -e "^$origin_regexp_esc$" -e "^$origin_replace_regexp_esc$" "${DBDIR}/taboo.all.list" 2> /dev/null
+		then
+			message_echo "INFO: Restoration of a conflict, $origin ($pkg), is avoided because it is taboo."
+			continue
+		fi
 		message_echo "INFO: Restoring a conflict, $origin ($pkg)."
 		if ! backup_pkg=`pkgsys_get_backup_pkg "$origin"`
 		then
@@ -255,7 +301,7 @@ reinstall_restore_conflicts ()
 # ============= Get the all requirements ready for a port by restarting them if deinstalled =============
 reinstall_chk_and_restore_requirements ()
 {
-	local tmp_restore
+	local tmp_restore tmp_isfailed
 	tmp_restore=${TMPDIR}/reinstall_setup_requirements:restore
 	tmp_isfailed=${TMPDIR}/reinstall_setup_requirements:isfailed
 	cp /dev/null "$tmp_restore"
@@ -272,7 +318,7 @@ reinstall_chk_and_restore_requirements ()
 			! pkgarc=`pkgsys_get_backup_pkg "$origin_orig"`
 		then
 			origin_regexp=`str_escape_regexp "$origin"`
-			if grep -m 1 -E "^$origin_regexp$" "${DBDIR}/failed.list" > /dev/null 2>&1
+			if grep -q -E "^$origin_regexp$" "${DBDIR}/failed.list" 2> /dev/null
 			then
 				touch "$tmp_isfailed"
 				break
@@ -285,6 +331,11 @@ reinstall_chk_and_restore_requirements ()
 	do
 		pkg=`pkgsys_pkgarc_to_pkgname "$pkgarc"`
 		pkg_regexp_esc=`str_escape_regexp "$pkg"`
+		if reinstall_chk_forbidden_conflicts "$pkg"
+		then
+			message_echo "INFO: Restoration of a conflict, $origin ($pkg), is avoided because it conflicts with installed packages."
+			continue
+		fi
 		message_echo "INFO: Restoring a backed-up requirement, $origin ($pkg)."
 		if pkg_add_fF "$pkgarc"
 		then
@@ -293,8 +344,6 @@ reinstall_chk_and_restore_requirements ()
 			mv "${DBDIR}/deleted_conflicts.tmp" "${DBDIR}/deleted_conflicts"
 		else
 			message_echo "WARNING: Failed to restore by the backed-up package." >&2
-			touch "$tmp_isfailed"
-			break
 		fi
 		
 	done < $tmp_restore
@@ -304,16 +353,20 @@ reinstall_chk_and_restore_requirements ()
 # ============= Back-up of the currently installed package =============
 reinstall_pkg_backup ()
 {
-	local backup_pkg
+	local backup_pkg pkg
 	reinstall_chk_stage_complete PKG_BACKUP && return
-	if [ -n "$REINSTALL_CURRENTPKG" ]
+	pkg=`echo "$REINSTALL_CURRENTPKG" | tr ' ' '\n' | grep -v '^$' | tail -n 1`
+	if [ -n "$pkg" ]
 	then
 		message_echo "-- (Creating temporary backup package for $REINSTALL_ORIGPKGTAG)"
-		backup_pkg=`pkgsys_create_backup_pkg "$REINSTALL_CURRENTPKG" "${DBDIR}/backup_packages"` || \
+		if backup_pkg=`pkgsys_create_backup_pkg "$pkg" "${DBDIR}/backup_packages"`
+		then
+			echo "$pkg" > $REINSTALL_DBNODE_DIR/backedup_version
+			echo "$backup_pkg" > $REINSTALL_DBNODE_DIR/backedup_pkgfile
+		else
 			message_echo "WARNING: Failed to create the backup package, but ignored by hoping success." >&2
-		cd "${PORTSDIR}/$REINSTALL_ORIGIN"
-		echo "$REINSTALL_CURRENTPKG" > $REINSTALL_DBNODE_DIR/backedup_version
-		echo "$backup_pkg" > $REINSTALL_DBNODE_DIR/backedup_pkgfile
+			return 1
+		fi
 	fi
 	reinstall_register_stage_complete PKG_BACKUP
 }
@@ -323,10 +376,13 @@ reinstall_deinstall ()
 {
 	[ -n "$REINSTALL_CURRENTPKG" ] || return 0
 	reinstall_chk_stage_complete DEINSTALL && return
-	pkg_delete_f "$REINSTALL_CURRENTPKG" || \
-	{
-		message_echo "WARNING: Failed to deinstall $REINSTALL_CURRENTPKG by $PKGSYS_CMD_PKG_DELETE." >&2
-	}
+	echo "$REINSTALL_CURRENTPKG" | tr ' ' '\n' | grep -v '^$' | while read pkg
+	do
+		pkg_delete_f "$pkg" || \
+		{
+			message_echo "WARNING: Failed to deinstall $pkg by $PKGSYS_CMD_PKG_DELETE." >&2
+		}
+	done
 	message_echo "-- (Trying to deinstall by ports to make sure. This usually ends up with warnings.)"
 	reinstall_make deinstall || \
 	{
@@ -338,7 +394,7 @@ reinstall_deinstall ()
 # ============= Recovery after failure of installation of the new package =============
 reinstall_failed_install_recover ()
 {
-	local backedup_version backup_pkg
+	local backedup_version origin_regexp_esc backup_pkg
 	reinstall_chk_stage_complete FAILED_INSTALL.RECOVER && return
 	message_echo "INFO: Trying to deinstall the failed/terminated installation (Ignore failures)."
 	if [ -n "$REINSTALL_CURRENTPKG" ]
@@ -348,16 +404,25 @@ reinstall_failed_install_recover ()
 	message_echo "INFO: Trying to deinstall by ports to make sure (This usually ends up with warnings)."
 	reinstall_make deinstall || :
 	backedup_version=`cat "$REINSTALL_DBNODE_DIR/backedup_version" 2> /dev/null || :`
-	if [ -n "$backedup_version" ]
+	origin_regexp_esc=`str_escape_regexp "$REINSTALL_ORIGIN"`
+	if grep -q -E "^$origin_regexp_esc$" "${DBDIR}/taboo.all.list" 2> /dev/null
 	then
-		message_echo "INFO: Restoring the backup of $backedup_version."
-		backup_pkg=`cat "$REINSTALL_DBNODE_DIR/backedup_pkgfile" 2> /dev/null || :`
-		if [ -z "$backup_pkg" -o ! -e "$backup_pkg" ]
+		message_echo "INFO: Restoration of the backup of $backedup_version is avoided because it is taboo."
+	elif [ -n "$backedup_version" ]
+	then
+		if reinstall_chk_forbidden_conflicts "$backedup_version"
 		then
-			message_echo "WARNING: No backup exists, gave up." >&2
-		elif ! pkg_add_fF "$backup_pkg"
-		then
-			message_echo "WARNING: Failed to restore $backedup_version. Note that your system may experience troubles by this error." >&2
+			message_echo "INFO: Restoration of the backup of $backedup_version, is avoided because it conflicts with installed packages."
+		else
+			message_echo "INFO: Restoring the backup of $backedup_version."
+			backup_pkg=`cat "$REINSTALL_DBNODE_DIR/backedup_pkgfile" 2> /dev/null || :`
+			if [ -z "$backup_pkg" -o ! -e "$backup_pkg" ]
+			then
+				message_echo "WARNING: No backup exists, gave up." >&2
+			elif ! pkg_add_fF "$backup_pkg"
+			then
+				message_echo "WARNING: Failed to restore $backedup_version. Note that your system may experience troubles by this error." >&2
+			fi
 		fi
 	fi
 	reinstall_register_stage_complete FAILED_INSTALL.RECOVER
@@ -394,6 +459,7 @@ reinstall_closing_operations_after_successful_install ()
 	local tag level initial_orig 
 	reinstall_chk_stage_complete CLOSING_OPERATIONS_AFTER_SUCCESSFUL_INSTALL && return
 	database_build_update_pkgname "$REINSTALL_ORIGIN" > /dev/null
+	database_build_update_pkgtag "$REINSTALL_ORIGIN"
 	for tag in all run build none
 	do
 		for level in full direct
@@ -408,7 +474,7 @@ reinstall_closing_operations_after_successful_install ()
 	initial_orig=`cat "$REINSTALL_DBNODE_DIR/initial_orig" 2> /dev/null` || :
 	[ -n "$initial_orig" ] && pkgsys_delete_backup_pkg "$initial_orig"
 	rm -f "$REINSTALL_DBNODE_DIR/backedup_pkgfile"
-	reinstall_deregister_stage in_install
+	reinstall_deregister_stage in_postinstall
 	database_record_success "$REINSTALL_ORIGIN"
 	temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
 	message_echo "===>  Done successfully"
@@ -420,8 +486,9 @@ reinstall_exec ()
 {
 	( set -e
 	REINSTALL_ORIGIN=$1
+	REINSTALL_ORIGIN_REGEXP=`str_escape_regexp "$REINSTALL_ORIGIN"`
 	REINSTALL_DBNODE_DIR=${DBDIR}/requires/$REINSTALL_ORIGIN
-	REINSTALL_CURRENTPKG=`database_build_update_pkgname "$REINSTALL_ORIGIN"`
+	REINSTALL_CURRENTPKG=`database_build_update_pkgname "$REINSTALL_ORIGIN" | tr '\n' ' ' | sed 's/ *$//'`
 	REINSTALL_IS_CURRENTPKG_LATEST=no
 	database_build_is_currentpkg_latest "$REINSTALL_ORIGIN" && REINSTALL_IS_CURRENTPKG_LATEST=yes
 	database_build_update_pkgtag "$REINSTALL_ORIGIN"
@@ -437,56 +504,64 @@ reinstall_exec ()
 		return
 	fi
 	reinstall_skip_if_in_a_list 'Skipped because being a leaf port' leaf_ports_to_delete && return
-	if [ -e "${DBDIR}/target_all" ]
+	if expr "$REINSTALL_CURRENTPKG" : '.* .*' > /dev/null
 	then
-		if [ $REINSTALL_IS_CURRENTPKG_LATEST = yes ]
-		then
-			reinstall_restore_if_temporarily_deinstalled
-			reinstall_skip 'Skipped because being already latest' "$REINSTALL_ORIGIN"
-			return
-		fi
-	elif [ ! -e "$REINSTALL_DBNODE_DIR/conf_updated" ]
+		message_echo "WARNING: Multiple packages are registered for this port. Reinstallation is needed to fix it." >&2
+	elif grep -q -E "^$REINSTALL_ORIGIN_REGEXP$" "${DBDIR}/damaged_package" 2> /dev/null
 	then
-		if [ -e "$nodedir/installed_by_pkg" ] && database_query_is_default_conf "$REINSTALL_ORIGIN" quiet
+		message_echo "WARNING: Installed files have invalid checksums for this port. Reinstallation is needed to fix it." >&2
+	else
+		if [ -e "${DBDIR}/target_all" ]
 		then
-			reinstall_restore_if_temporarily_deinstalled
-			reinstall_skip 'Skipped because already upgraded with a prebuilt package'
-			return
-		fi
-		reinstall_skip_if_in_a_list 'Skipped because the reinstallation has been already completed' \
-			"success.${REINSTALL_DBSUFFIX}.list" restore && return
-		if [ $opt_skip_unchanged = yes ]
-		then
-			if [ ! -e "$REINSTALL_DBNODE_DIR/necessary_upgrade.${REINSTALL_DBSUFFIX}" ]
+			if [ $REINSTALL_IS_CURRENTPKG_LATEST = yes ]
 			then
 				reinstall_restore_if_temporarily_deinstalled
-				if [ -e "$REINSTALL_DBNODE_DIR/installed_version" ]
-				then
-					reinstall_skip 'Skipped because being already latest as well as the all requirements'
-				else
-					reinstall_skip 'Skipped because being an only-build-time dependency of already latest packages'
-				fi
+				reinstall_skip 'Skipped because being already latest' "$REINSTALL_ORIGIN"
 				return
 			fi
-		fi
-		if [ \( $opt_skip_unchanged = no -a -e "$REINSTALL_DBNODE_DIR/succeeded_once" \) \
-			-o \( $opt_skip_unchanged = yes -a $REINSTALL_IS_CURRENTPKG_LATEST = yes \) ]
+		elif [ ! -e "$REINSTALL_DBNODE_DIR/conf_updated" ]
 		then
-			if [ ! -e "$REINSTALL_DBNODE_DIR/need_reinstall_due_to_upgraded_requirements.${REINSTALL_DBSUFFIX}" ]
+			if [ -e "$nodedir/installed_by_pkg" ] && database_query_is_default_conf "$REINSTALL_ORIGIN" quiet
 			then
-				if ! database_query_is_necessary_upgrade "$REINSTALL_ORIGIN"
+				reinstall_restore_if_temporarily_deinstalled
+				reinstall_skip 'Skipped because already upgraded with a prebuilt package'
+				return
+			fi
+			reinstall_skip_if_in_a_list 'Skipped because the reinstallation has been already completed' \
+				"success.${REINSTALL_DBSUFFIX}.list" restore && return
+			if [ $opt_skip_unchanged = yes ]
+			then
+				if [ ! -e "$REINSTALL_DBNODE_DIR/necessary_upgrade.${REINSTALL_DBSUFFIX}" ]
 				then
 					reinstall_restore_if_temporarily_deinstalled
-					fileedit_add_a_line_if_new "$REINSTALL_ORIGIN" \
-						"${DBDIR}/todo_after_requirements_succeed.${REINSTALL_DBSUFFIX}.list"
-					reinstall_skip 'Skipped because being already latest or failed as well as the all requirements'
+					if [ -e "$REINSTALL_DBNODE_DIR/installed_version" ]
+					then
+						reinstall_skip 'Skipped because being already latest as well as the all requirements'
+					else
+						reinstall_skip 'Skipped because being an only-build-time dependency of already latest packages'
+					fi
 					return
 				fi
 			fi
+			if [ \( $opt_skip_unchanged = no -a -e "$REINSTALL_DBNODE_DIR/succeeded_once" \) \
+				-o \( $opt_skip_unchanged = yes -a $REINSTALL_IS_CURRENTPKG_LATEST = yes \) ]
+			then
+				if [ ! -e "$REINSTALL_DBNODE_DIR/need_reinstall_due_to_upgraded_requirements.${REINSTALL_DBSUFFIX}" ]
+				then
+					if ! database_query_is_necessary_upgrade "$REINSTALL_ORIGIN"
+					then
+						reinstall_restore_if_temporarily_deinstalled
+						fileedit_add_a_line_if_new "$REINSTALL_ORIGIN" \
+							"${DBDIR}/todo_after_requirements_succeed.${REINSTALL_DBSUFFIX}.list"
+						reinstall_skip 'Skipped because being already latest or failed as well as the all requirements'
+						return
+					fi
+				fi
+			fi
 		fi
+		reinstall_skip_if_in_a_list 'Marked to be manually-done' manually_done.list restore && return
 	fi
 	reinstall_skip_if_in_a_list 'Skipped because being a hold package' conf/HOLD:PORTS.parsed restore && return
-	reinstall_skip_if_in_a_list 'Marked to be manually-done' manually_done.list restore && return
 	if database_query_is_a_port_suppressed "$REINSTALL_ORIGIN"
 	then
 		reinstall_restore_if_temporarily_deinstalled
@@ -534,23 +609,35 @@ reinstall_exec ()
 		else
 			reinstall_register_stage in_bypkg
 		fi
-		reinstall_pkg_backup
-		reinstall_deinstall
 		if ! reinstall_chk_stage_complete INSTALL_BY_PKG
 		then
 			if ! reinstall_chk_stage FAILED_INSTALL_BY_PKG
 			then
+				func_pkg_inst_remote_fetch=pkg_inst_remote_fetch
 				func_pkg_inst_remote=pkg_inst_remote
-				[ $opt_use_legacy_pkg_for_missing_pkgng = yes ] && \
-					func_pkg_inst_remote=pkg_inst_remote_wild
-				if $func_pkg_inst_remote "$REINSTALL_NEWPKGNAME"
+				if [ $opt_use_legacy_pkg_for_missing_pkgng = yes ]
 				then
-					reinstall_register_stage_complete INSTALL_BY_PKG
+					func_pkg_inst_remote_fetch=pkg_inst_remote_wild_fetch
+					func_pkg_inst_remote=pkg_inst_remote_wild
+				fi
+				if $func_pkg_inst_remote_fetch "$REINSTALL_NEWPKGNAME"
+				then
+					pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN" \
+						| reinstall_backup_and_delete_conflicts
+					reinstall_pkg_backup || :
+					reinstall_register_stage in_add_pkg
+					reinstall_deinstall
+					if $func_pkg_inst_remote "$REINSTALL_NEWPKGNAME"
+					then
+						reinstall_register_stage_complete INSTALL_BY_PKG
+					else
+						reinstall_register_stage FAILED_INSTALL_BY_PKG
+					fi
 				else
 					reinstall_register_stage FAILED_INSTALL_BY_PKG
 				fi
 			fi
-			if reinstall_chk_stage FAILED_INSTALL_BY_PKG
+			if reinstall_chk_stage FAILED_IN_ADD_PKG && reinstall_chk_stage in_add_pkg
 			then
 				reinstall_failed_install_recover
 			fi
@@ -568,14 +655,12 @@ reinstall_exec ()
 		reinstall_deregister_stage in_bypkg
 		message_echo "WARNING: (Re)installation-by-package is unsuccessful, so retrying by using port." >&2
 	fi
-	if reinstall_chk_stage in_build
+	if reinstall_chk_stage in_prebuild
 	then
-		message_echo "(Restarting the previously terminated build process...)"
-		flag_restarted_build=yes
+		message_echo "(Restarting the previously terminated pre-build process...)"
 	else
 		reinstall_restore_conflicts
-		reinstall_register_stage in_build
-		flag_restarted_build=no
+		reinstall_register_stage in_prebuild
 	fi
 	if [ -e "$REINSTALL_DBNODE_DIR/BEFOREBUILD.conf" ] && ! reinstall_chk_stage_complete BEFOREBUILD
 	then
@@ -635,6 +720,7 @@ reinstall_exec ()
 		fi
 		reinstall_register_stage_complete FETCH
 	fi
+	reinstall_deregister_stage in_prebuild
 	if [ $opt_batch_ports_only = yes -o $opt_interactive_ports_only = yes ]
 	then
 		to_skip=no
@@ -657,23 +743,43 @@ reinstall_exec ()
 			return
 		fi
 	fi
+	if reinstall_chk_stage in_build
+	then
+		message_echo "(Restarting the previously terminated build process...)"
+		flag_restarted_build=yes
+	else
+		reinstall_register_stage in_build
+		flag_restarted_build=no
+	fi
 	if ! reinstall_chk_stage_complete BUILD
 	then
+		pkgsys_get_conflicting_installed_pkgs build "$REINSTALL_ORIGIN" \
+			| reinstall_backup_and_delete_conflicts
 		if ! reinstall_make build
 		then
-			if pkgsys_get_conflicting_pkgs build "$REINSTALL_ORIGIN" \
-				> ${TMPDIR}/reinstall_exec::conflicts_build
+			[ $flag_restarted_build = yes ] && message_echo "INFO: This failure may be due to restarting from a terminated build."
+			is_build_env_modified=no
+			if pkg_info_eO "$REINSTALL_ORIGIN"
 			then
-				reinstall_backup_and_delete_conflicts < ${TMPDIR}/reinstall_exec::conflicts_build
-			elif [ $flag_restarted_build = no ]
+				message_echo "INFO: The currently installed package for this port is deinstalled for retrial because it may have effects on build."
+				reinstall_pkg_backup || :
+				reinstall_deinstall
+				is_build_env_modified=yes
+			fi
+			if pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN" \
+				> ${TMPDIR}/reinstall_exec::conflicts_install
+			then
+				message_echo "INFO: Install-only conflicts are deinstalled for retrial because they may have effects on build."
+				reinstall_backup_and_delete_conflicts < ${TMPDIR}/reinstall_exec::conflicts_install
+				is_build_env_modified=yes
+			fi
+			if [ $flag_restarted_build = no -a $is_build_env_modified = no ]
 			then
 				reinstall_restore_conflicts "$REINSTALL_ORIGIN"
 				reinstall_restore_if_temporarily_deinstalled
 				reinstall_errproc 'build'
 				temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
 				return
-			else
-				message_echo "INFO: This failure may be due to restarting from a terminated build."
 			fi
 			reinstall_deregister_stage_complete CLEAN_BEFORE_BUILD
 			message_echo "INFO: Retrying the build process after cleaning for $REINSTALL_ORIGPKGTAG."
@@ -685,6 +791,7 @@ reinstall_exec ()
 				temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
 				return
 			}
+			reinstall_register_stage_complete CLEAN_BEFORE_BUILD
 			reinstall_make build || \
 			{
 				reinstall_restore_conflicts "$REINSTALL_ORIGIN"
@@ -697,13 +804,15 @@ reinstall_exec ()
 		reinstall_register_stage_complete BUILD
 	fi
 	reinstall_deregister_stage in_build
-	if reinstall_chk_stage in_instal
+	if reinstall_chk_stage in_install
 	then
 		message_echo "(Restarting the previously terminated installation process...)"
+		flag_restarted_build=yes
 	else
 		reinstall_register_stage in_install
+		flag_restarted_build=no
 	fi
-	reinstall_pkg_backup
+	reinstall_pkg_backup || :
 	if [ -e "$REINSTALL_DBNODE_DIR/BEFOREDEINSTALL.conf" ] && ! reinstall_chk_stage_complete BEFOREDEINSTALL
 	then
 		if [ -n "$REINSTALL_CURRENTPKG" ]
@@ -721,6 +830,8 @@ reinstall_exec ()
 		fi
 		reinstall_register_stage_complete BEFOREDEINSTALL
 	fi
+	pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN" \
+		| reinstall_backup_and_delete_conflicts
 	reinstall_deinstall
 	if ! reinstall_chk_stage_complete INSTALL
 	then
@@ -737,17 +848,6 @@ reinstall_exec ()
 			then
 				touch "$REINSTALL_DBNODE_DIR/installed_timestamp"
 				reinstall_register_stage_complete INSTALL
-			elif pkgsys_get_conflicting_pkgs install "$REINSTALL_ORIGIN" \
-				> ${TMPDIR}/reinstall_exec::conflicts_install
-			then
-				reinstall_backup_and_delete_conflicts < ${TMPDIR}/reinstall_exec::conflicts_install
-				message_echo "INFO: Retrying to the installation process for $REINSTALL_ORIGPKGTAG."
-				if reinstall_make $insttarget
-				then
-					reinstall_register_stage_complete INSTALL
-				else
-					reinstall_register_stage FAILED_INSTALL
-				fi
 			else
 				reinstall_register_stage FAILED_INSTALL
 			fi
@@ -791,6 +891,8 @@ reinstall_exec ()
 		message_echo "-- AFTERINSTALL operations (end)"
 		reinstall_register_stage_complete AFTERINSTALL
 	fi
+	reinstall_deregister_stage in_install
+	reinstall_register_stage in_postinstall
 	reinstall_restore_conflicts "$REINSTALL_ORIGIN"
 	if ! reinstall_chk_stage_complete CLEAN_AFTER_INSTALL
 	then
