@@ -53,7 +53,7 @@ str_convert_portsglob_to_regexp_pattern ()
 # ============= Compose a part of sentence by concatenating items in a list as a logical product =============
 str_linearize_list_and ()
 {
-	echo "$*" | sed -E 's/^ +//; s/ +$//; s/ +/, /; s/, ([^ ]+)$/ and \1/'
+	echo "$*" | sed -E 's/^ +//; s/ +$//; s/ +/, /g; s/, ([^ ]+)$/ and \1/'
 }
 
 # ============= Manipulate an origin name from a diretory path =============
